/* Copyright (c) 2006-2019, Universities Space Research Association (USRA).
*  All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the Universities Space Research Association nor the
*       names of its contributors may be used to endorse or promote products
*       derived from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY USRA ``AS IS'' AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL USRA BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
* OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

// Definition of the Socket class

#ifndef Socket_class
#define Socket_class

#include "plexil-config.h"

#include <string>
#include <iostream>

#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h> // sockaddr_in
#endif

#include "SocketException.h"

const int MAXHOSTNAME = 200;
const int MAXCONNECTIONS = 5;
const int MAXRECV = 500;
const int SOCK_MSG_NOSIGNAL = 0;

class Socket
{
 public:
  Socket();
  virtual ~Socket();

  // operators
  const Socket& operator << ( const std::string& ) const;
  const Socket& operator << ( const char& ) const;
  const Socket& operator >> ( std::string& ) const;

  // Server initialization
  bool create();
  bool bind ( const uint16_t port );
  bool listen() const;
  virtual bool accept ( Socket& ) const;

  // Client initialization
  bool connect ( const std::string host, const uint16_t port );

  // Data Transimission
  bool send ( const std::string ) const;
  bool send (char* msg, size_t size ) const;
  int recv ( std::string& ) const;


  void set_non_blocking ( const bool );

  bool is_valid() const { return m_sock != -1; }
  //bool isOpen() const;

 private:

  int m_sock;
  sockaddr_in m_addr;


};


#endif
